<?php
/**
 * PlaySmsSendSms Class Doc Comment
 *
 * @author   App World sp. z o.o.
 * @copyright 2018 App World sp. z o.o.
 * @license  http://www.gnu.org/copyleft/gpl.html GNU General Public License
 * @category Class
 * @package  playsmsusernotifier
 */

class PlaySmsSendSms
{
    /**
     *
     * Sends text message through PlaySMS API
     *
     * @param string $apiUserName Api Key/Username for PlaySMS Api
     * @param string $apiPassword Password for PlaySMS Api
     * @param string $to Number to which send SMS
     * @param string $message Message to send
     * @param string $header Specific text message header, provided by PlaySMS Api
     * @param int $clearPolish Whether to change polish special characters (and all special characters)
     * to normal characters, possible values 0/1
     * @param int $test Test mode gives only callback, doesnt send SMS, possible values 0/1
     *
     * @return string method send() returns "OK" on success or "ERROR" on failure
     */

    public $apiUserName;
    public $apiPassword;
    public $to = null;
    public $message = null;
    public $header = null;
    public $clearPolish;
    public $test = 0;

    public function __construct($apiUserName, $apiPassword, $to, $message, $test, $header, $clearPolish)
    {
        $this->setTo($to);
        $this->setMessage($message);
        $this->setApiUserName($apiUserName);
        $this->setApiPassword($apiPassword);
        $this->setTest($test);
        $this->setHeader($header);
        $this->clearPolish($clearPolish);
    }

    public function setApiUserName($apiUserName)
    {
        $this->apiUserName = (string)$apiUserName;
    }

    public function setHeader($header)
    {
        if ($header !== null) {
            $this->header = (string)$header;
        }
    }

    public function setApiPassword($apiPassword)
    {
        $this->apiPassword = (string)$apiPassword;
    }

    public function setResponseFormat($format)
    {
        $this->format = (string)$format;
    }

    public function setTo($to)
    {
        $this->to = (string)$to;
    }

    public function setMessage($message)
    {
        $this->message = (string)$message;
    }

    public function setTest($test)
    {
        $this->test = (int)$test;
    }

    public function clearPolish($clearPolish)
    {
        $this->clearPolish = (int)$clearPolish;
    }

    public function send()
    {
        $mainDataXMLDoc = new SimpleXMLElement("<?xml version=\"1.0\" encoding=\"UTF-8\"?><data/>");
        $mainDataXMLDoc->addChild('key', $this->apiUserName);
        $mainDataXMLDoc->addChild('password', $this->apiPassword);
        $sms = $mainDataXMLDoc->addChild('sms');
        $sms->addChild('format', $this->format);
        $sms->addChild('msg', $this->message);
        $sms->addChild('from', $this->header);
        $sms->addChild('to', $this->to);
        $sms->addChild('clear_polish', $this->clearPolish);
        $sms->addChild('test', $this->test);

        $xmlObjectToString = (string)$mainDataXMLDoc->asXML();


        $xmlData = "xmldata=" . $xmlObjectToString . "";

        $url = 'https://api.smsplanet.pl';

        $c = curl_init();
        curl_setopt($c, CURLOPT_URL, $url);
        curl_setopt($c, CURLOPT_HEADER, false);
        curl_setopt($c, CURLOPT_POST, true);
        curl_setopt($c, CURLOPT_POSTFIELDS, $xmlData);
        curl_setopt($c, CURLOPT_RETURNTRANSFER, true);

        $content = curl_exec($c);

        curl_close($c);

        $responseHeading = $this->playSmsGetStringBetween($content, '<result>', '</result>');

        if ((string)$responseHeading === 'ERROR') {
            $errorMessage = $this->playSmsGetStringBetween($content, '<errorMsg>', '</errorMsg>');
            $this->logError($errorMessage);
        }

        return $content;
    }

    private function logError($errorMessage)
    {
        $date         = new DateTime();
        $dateToInsert = $date->format('Y-m-d H:i:s');
        Db::getInstance()->insert('playsms_errors', array(
            'error_date' => pSQL($dateToInsert),
            'message'    => pSQL($errorMessage),
        ));

        return $errorMessage;
    }

    private function playSmsGetStringBetween($string, $start, $end)
    {
        $string = ' ' . $string;
        $ini    = strpos($string, $start);
        if ($ini == 0) {
            return '';
        }
        $ini += Tools::strlen($start);
        $len = strpos($string, $end, $ini) - $ini;

        return Tools::substr($string, $ini, $len);
    }
}
