<?php
/**
 * playsmsusernotifier File Doc Comment
 *
 * Module sends notifying sms messages after user created events on status change,
 * new order, changing address of user.
 *
 * @author   App World Sp. z o.o.
 * @copyright 2018 App World Sp. z o.o.
 * @license  http://www.gnu.org/copyleft/gpl.html GNU General Public License
 * @category playsmsusernotifier
 * @package  playsmsusernotifier
 * @version 1.0.0
 *
 */

if (! defined('_PS_VERSION_')) {
    exit;
}

require_once(dirname(__FILE__) . '/classes/PlaySmsSenderFields.php');
require_once(dirname(__FILE__) . '/classes/PlaySmsSendSms.php');

class PlaySmsUserNotifier extends Module
{

    public function __construct()
    {

        $this->name                   = 'playsmsusernotifier';
        $this->tab                    = 'others';
        $this->version                = '1.0.0';
        $this->author                 = 'App World Sp. z o.o.';
        $this->need_isntance          = 0;
        $this->ps_versions_compliancy = array('min' => '1.7', 'max' => '1.8');
        $this->bootstrap              = true;

        parent::__construct();

        $this->displayName      = $this->l('SMSPLANET');
        $this->description      = $this->l('Moduł pozwala na informowanie użytkownika wiadomością 
        SMS po spełnieniu określonej akcji na stronie.');
        $this->confirmUninstall = $this->l('Czy jesteś pewien, że chcesz usunąć wtyczkę SMSPLANET?');


        if (! Configuration::get('API_KEY')) {
            $this->warning = $this->l('Brak klucza API SMSPLANET.');
        }

        if (! Configuration::get('API_PASSWORD')) {
            $this->warning = $this->l('Brak hasła API SMSPLANET.');
        }

        if (! Configuration::get('SENDER_FIELD')) {
            $this->warning = $this->l('Brak pobranych pól nadawcy.');
        }
    }

    public function install()
    {

        if (Shop::isFeatureActive()) {
            Shop::setContext(Shop::CONTEXT_ALL);
        }

        if (! parent::install() ||
            ! $this->registerHook('header') ||
            ! $this->registerHook('newOrder') ||
            ! $this->registerHook('actionOrderStatusPostUpdate') ||
            ! $this->registerHook('actionValidateCustomerAddressForm') ||
            ! $this->createDatabaseTables()
        ) {
            return false;
        }

        return true;
    }

    private function createDatabaseTables()
    {

        $eventsTableCreationQuery = "CREATE TABLE IF NOT EXISTS `" . _DB_PREFIX_ . "playsms_events` (
          id MEDIUMINT(9) NOT NULL AUTO_INCREMENT,
          name  VARCHAR(55) NOT NULL,
          associated_hook  VARCHAR(55) NOT NULL,
          sms_message  VARCHAR(500) NOT NULL,
          action_type  VARCHAR(255) NOT NULL,
          convert_characters  TINYINT(1) NULL,
          message_header  VARCHAR(255) NOT NULL,
          PRIMARY KEY  (id)
        ) ENGINE=" . _MYSQL_ENGINE_ . " DEFAULT CHARSET=utf8;";

        $createEventsTable = Db::getInstance()->execute($eventsTableCreationQuery);

        $errorTableCreationQuery = "CREATE TABLE IF NOT EXISTS `" . _DB_PREFIX_ . "playsms_errors` (
          id MEDIUMINT(9) NOT NULL AUTO_INCREMENT,
          message VARCHAR(255) NOT NULL,
          error_date DATETIME,
          PRIMARY KEY  (id)
        ) ENGINE=" . _MYSQL_ENGINE_ . " DEFAULT CHARSET=utf8;";
        $createErrorTable        = Db::getInstance()->execute($errorTableCreationQuery);


        if (! $createEventsTable ||
            ! $createErrorTable) {
            return false;
        }

        return true;
    }


    public function uninstall()
    {

        $uninstallEventsTable = "DROP TABLE IF EXISTS `" . _DB_PREFIX_ . "playsms_events`;";
        $uninstallErrorsTable = "DROP TABLE IF EXISTS `" . _DB_PREFIX_ . "playsms_errors`;";

        if (! parent::uninstall() ||
            ! Configuration::deleteByName('API_KEY') ||
            ! Configuration::deleteByName('API_PASSWORD') ||
            ! Configuration::deleteByName('SENDER_FIELD') ||
            ! Configuration::deleteByName('CONVERT_CHARACTERS')

        ) {
            return false;
        }

        if (! Db::getInstance()->execute($uninstallEventsTable)) {
            return false;
        }

        if (! Db::getInstance()->execute($uninstallErrorsTable)) {
            return false;
        }

        return true;
    }

    public function getContent()
    {
        $output = null;
        $this->context->controller->addCSS($this->_path . 'views/css/playsmsusernotifier.css', 'all');
        $this->context->controller->addJS($this->_path . 'views/js/playsmsusernotifier.js', 'all');

        if (Tools::isSubmit('submit' . $this->name)) {
            $api_key              = (string)Tools::getValue('API_KEY');
            $api_password         = (string)Tools::getValue('API_PASSWORD');
            $api_sender_field     = (string)Tools::getValue('SENDER_FIELD');
            $apiConvertCharacters = (int)(Tools::getValue('CONVERT_CHARACTERS'));
            $response             = 1;

            if (! $api_key
                || empty($api_key)
                || ! Validate::isString($api_key)
                || ! $this->validateUUIDv4($api_key)) {
                $response = 0;
                $output   .= $this->displayError($this->l('Nieprawidłowy klucz API.'));
            } else {
                Configuration::updateValue('API_KEY', $api_key);
            }

            if (! $api_password
                || empty($api_password)
                || ! Validate::isString($api_password)) {
                $response = 0;
                $output   .= $this->displayError($this->l('Nieprawidłowe hasło API.'));
            } else {
                Configuration::updateValue('API_PASSWORD', $api_password);
            }

            if (! $api_sender_field
                || empty($api_sender_field)
                || ! Validate::isString($api_sender_field)
            ) {
                $response = 0;
                $output   .= $this->displayError($this->l('Nieprawidłowe pole nadawcy.'));
            } else {
                Configuration::updateValue('SENDER_FIELD', $api_sender_field);
            }

            if ($response === 1) {
                $output .= $this->displayConfirmation($this->l('Ustawienia zapisane.'));
                Configuration::updateValue('CONVERT_CHARACTERS', $apiConvertCharacters);
            }
        }

        return $output . $this->displayForm() . $this->displayEvents() . $this->displayErrorsLog();
    }

    public function displayEvents()
    {

        $default_sender_header = Configuration::get('SENDER_FIELD');

        $loader = new Twig_Loader_Filesystem(dirname(__FILE__) . '/views/templates/admin');
        $twig   = new Twig_Environment($loader);

        $template = $twig->load('events-container.twig');

        return $template->render(array(
            'playsms_sender_fields'         => $this->renderSenderFields(),
            'playsms_increment_value'       => 1,
            'playsms_default_sender_header' => $default_sender_header,
            'playsms_order_states'          => $this->getAvailableOrderStates(),
        ));
    }

    public function displayErrorsLog()
    {

        $loader = new Twig_Loader_Filesystem(dirname(__FILE__) . '/views/templates/admin');
        $twig   = new Twig_Environment($loader);

        $template = $twig->load('errorlog.twig');

        return $template->render(array(
            'playsms_errors' => $this->getErrors()
        ));
    }

    public function getErrors()
    {

        $this->deleteOldErrors();
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from('playsms_errors');
        $sql->orderby('id', 'asc');
        $errors = Db::getInstance()->executeS($sql);

        return $errors;
    }

    public function deleteOldErrors()
    {
        /**
         * If error older than 30 days = delete
         */
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from('playsms_errors');
        $errors = Db::getInstance()->executeS($sql);

        foreach ($errors as $error) {
            $errorDateTimeObject = new DateTime($error['error_date']);
            $errorDueDate        = $errorDateTimeObject->add(new DateInterval('P30D'));
            $nowDateTime         = new DateTime();
            if ($nowDateTime > $errorDueDate) {
                Db::getInstance()->delete('playsms_errors', 'id=' . (int)$error['id']);
            }
        }
    }

    public function renderSenderFields()
    {

        $api_key      = Configuration::get('API_KEY');
        $api_password = Configuration::get('API_PASSWORD');

        $getSenderFields = array();

        if (isset($api_key) && ! empty($api_key) && isset($api_password) && ! empty($api_password)) {
            $getSenderFieldsData = new PlaySmsSenderFields($api_key, $api_password);
            $getSenderFields     = $getSenderFieldsData->getHeaders();

            if ($getSenderFields === 'ERROR') {
                $this->logError($this->l('Błąd pobierania nagłówków, dane są niepoprawne.'));
            }

            $getSenderFields = str_replace("[", "", $getSenderFields);
            $getSenderFields = str_replace("]", "", $getSenderFields);
            $getSenderFields = str_replace('"', "", $getSenderFields);
            $getSenderFields = explode(",", $getSenderFields);
        } else {
            $this->logError($this->l('Błąd pobierania nagłówków, dane logowania API są niekompletne.'));
        }

        return $getSenderFields;
    }

    private function logError($errorMessage)
    {

        $date         = new DateTime();
        $dateToInsert = $date->format('Y-m-d H:i:s');

        Db::getInstance()->insert('playsms_errors', array(
            'error_date' => pSQL($dateToInsert),
            'message'    => pSQL($errorMessage),
        ));

        return $errorMessage;
    }

    public function displayForm()
    {

        $default_lang = Configuration::get('PS_LANG_DEFAULT');
        $api_key      = Configuration::get('API_KEY');
        $api_password = Configuration::get('API_PASSWORD');

        if (isset($api_key) && ! empty($api_key) && isset($api_password) && ! empty($api_password)) {
            $getSenderFieldsData = new PlaySmsSenderFields($api_key, $api_password);
            $getSenderFields     = $getSenderFieldsData->getHeaders();
        } else {
            $getSenderFields = 'EMPTY';
        }

        if ($getSenderFields === 'ERROR') {
            $options = array(
                array(
                    'id_option' => 0,
                    'name'      => 'Niepoprawny klucz bądź hasło API.'
                )
            );
        } elseif ($getSenderFields === 'EMPTY') {
            $options = array(
                array(
                    'id_option' => 0,
                    'name'      => 'Uzupełnij dane API i pobierz listę pól nadawcy.'
                )
            );
        } else {
            $options         = array();
            $getSenderFields = str_replace("[", "", $getSenderFields);
            $getSenderFields = str_replace("]", "", $getSenderFields);
            $getSenderFields = str_replace('"', "", $getSenderFields);
            $getSenderFields = explode(",", $getSenderFields);
            $i               = 1;
            foreach ($getSenderFields as $sender_field) {
                $element = array(
                    'id_option' => (int)$i,
                    'name'      => (string)$sender_field
                );
                array_push($options, $element);
                $i++;
            }
        }

        $fields_form = array(
            'form' => array(
                'legend'  => array(
                    'title' => $this->l('Ustawienia ogólne'),
                ),
                'input'   => array(
                    array(
                        'type'     => 'text',
                        'label'    => $this->l('Klucz API:'),
                        'name'     => 'API_KEY',
                        'size'     => 55,
                        'class'    => 'fixed-width-xxl',
                        'required' => true
                    ),
                    array(
                        'type'     => 'text',
                        'label'    => $this->l('Hasło API'),
                        'name'     => 'API_PASSWORD',
                        'size'     => 55,
                        'class'    => 'fixed-width-xxl',
                        'required' => true
                    ),
                    array(
                        'type'    => 'select',
                        'lang'    => true,
                        'label'   => $this->l('Domyślne pole nadawcy'),
                        'name'    => 'SENDER_FIELD',
                        'class'   => 'fixed-width-xxl',
                        'desc'    => $this->l(
                            'Kliknij przycisk znajdujący się w prawym dolnym rogu ustawień by zaktualizować listę.'
                        ),
                        'options' => array(
                            'query' => $options,
                            'id'    => 'id_option',
                            'name'  => 'name'
                        )
                    ),
                    array(
                        'type'    => 'switch',
                        'label'   => $this->l('Zamiana polskich znaków na ich odpowiedniki (domyślne)'),
                        'name'    => 'CONVERT_CHARACTERS',
                        'is_bool' => true,
                        'desc'    => $this->l(
                            'Domyślne ustawienia zamieniania polskich znaków na ich odpowiedniki np. ł na l.'
                        ),
                        'values'  => array(
                            array(
                                'id'    => 'convert_characters_on',
                                'value' => 1,
                            ),
                            array(
                                'id'    => 'convert_characters_off',
                                'value' => 0,
                            )
                        )
                    ),
                ),
                'buttons' => array(
                    'test' => array(
                        'type'  => 'submit',
                        'title' => $this->l('Aktualizuj listę pól nadawcy.'),
                        'name'  => 'SENDER_FIELD_BUTTON',
                        'class' => 'btn btn-default pull-right',
                        'icon'  => 'process-icon-save',
                    ),
                ),
                'submit'  => array(
                    'title' => $this->l('Zapisz ustawienia'),
                    'class' => 'btn btn-success pull-right'
                )
            )
        );

        $helper = new HelperForm();

        $helper->module          = $this;
        $helper->name_controller = $this->name;
        $helper->token           = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex    = AdminController::$currentIndex . '&configure=' . $this->name;

        // Language
        $helper->default_form_language    = $default_lang;
        $helper->allow_employee_form_lang = $default_lang;
        // Title and toolbar
        $helper->title          = $this->displayName;
        $helper->show_toolbar   = true;        // false -> remove toolbar
        $helper->toolbar_scroll = true;      // yes - > Toolbar is always visible on the top of the screen.
        $helper->submit_action  = 'submit' . $this->name;
        $helper->toolbar_btn    = array(
            'save' =>
                array(
                    'desc' => $this->l('Save'),
                    'href' => AdminController::$currentIndex . '&configure=' . $this->name . '&save' . $this->name .
                              '&token=' . Tools::getAdminTokenLite('AdminModules'),
                ),
            'back' => array(
                'href' => AdminController::$currentIndex . '&token=' . Tools::getAdminTokenLite('AdminModules'),
                'desc' => $this->l('Back to list')
            )
        );

        // Load current value
        $helper->fields_value['API_KEY']            = Configuration::get('API_KEY');
        $helper->fields_value['API_PASSWORD']       = Configuration::get('API_PASSWORD');
        $helper->fields_value['SENDER_FIELD']       = Configuration::get('SENDER_FIELD');
        $helper->fields_value['CONVERT_CHARACTERS'] = Configuration::get('CONVERT_CHARACTERS');

        return $helper->generateForm(array($fields_form));
    }

    public function hookNewOrder($params)
    {
        return $this->hookActionOrderStatusPostUpdate($params);
    }

    public function hookActionOrderStatusPostUpdate($params)
    {
        /**
         * Workaround to catch order creation (when status is created)
         */
        $newOrder     = $params['orderStatus']; // after order is placed
        $orderStatus  = $params['newOrderStatus'];  // after status changed
        $api_key      = Configuration::get('API_KEY');
        $api_password = Configuration::get('API_PASSWORD');
        $test         = 0;

        if (isset($orderStatus) && ! empty($orderStatus)) {
            $orderStatusID = $orderStatus->id;
            $order         = new Order($params['id_order']);
            $addressObject = new Address($order->id_address_delivery);
            $customerPhone = $addressObject->phone;

            $getCorrespondingEvent = $this->getCorrespondingToCurrentOrderStatusEvent($orderStatusID);

            if (isset($getCorrespondingEvent) && ! empty($getCorrespondingEvent)) {
                foreach ($getCorrespondingEvent as $correspondingEventData) {
                    $unconvertedSmsMessage = $correspondingEventData['sms_message'];
                    $convertedSmsMessage   = $this->convertSMSTextMessage($order, $unconvertedSmsMessage);
                    $convertCharacters     = $correspondingEventData['convert_characters'];
                    $messageHeader         = $correspondingEventData['message_header'];
                    $sendSMS               = new PlaySmsSendSms(
                        $api_key,
                        $api_password,
                        $customerPhone,
                        $convertedSmsMessage,
                        $test,
                        $messageHeader,
                        $convertCharacters
                    );
                    $sendSMS->send();
                }
            }
        } elseif (isset($newOrder) && ! empty($newOrder)) {
            $cart                 = $params['cart'];
            $addressObject        = new Address($cart->id_address_delivery);
            $order                = $params['order'];
            $customerPhone        = $addressObject->phone;
            $getNewOrderEventData = $this->getNewOrderStatus();

            if (isset($getNewOrderEventData) && ! empty($getNewOrderEventData) && is_array($getNewOrderEventData)) {
                foreach ($getNewOrderEventData as $correspondingEventData) {
                    $unconvertedSmsMessage = $correspondingEventData['sms_message'];
                    $convertedSmsMessage   = $this->convertSMSTextMessage($order, $unconvertedSmsMessage);
                    $convertCharacters     = $correspondingEventData['convert_characters'];
                    $messageHeader         = $correspondingEventData['message_header'];
                    $sendSMS               = new PlaySmsSendSms(
                        $api_key,
                        $api_password,
                        $customerPhone,
                        $convertedSmsMessage,
                        $test,
                        $messageHeader,
                        $convertCharacters
                    );
                    $sendSMS->send();
                }
            }
        }
    }

    public function hookActionValidateCustomerAddressForm($params)
    {

        $getAddressChangeEventData = $this->getAddressChangeEventData();

        if (isset($getAddressChangeEventData) && ! empty($getAddressChangeEventData)) {
            $api_key      = Configuration::get('API_KEY');
            $api_password = Configuration::get('API_PASSWORD');
            $test         = 0;

            foreach ($getAddressChangeEventData as $addressChangeData) {
                $unconvertedSmsMessage = (string)$addressChangeData['sms_message'];
                $convertCharacters     = (int)$addressChangeData['convert_characters'];
                $messageHeader         = (string)$addressChangeData['message_header'];
                $addressObject         = $params['form'];
                $customerPhone         = $addressObject->getField("phone")->getValue();

                if (isset($customerPhone) && ! empty($customerPhone)) {
                    $convertedMessage = $this->convertAddressSMSTextMessage($addressObject, $unconvertedSmsMessage);
                    $sendSMS          = new PlaySmsSendSms(
                        $api_key,
                        $api_password,
                        $customerPhone,
                        $convertedMessage,
                        $test,
                        $messageHeader,
                        $convertCharacters
                    );
                    $sendSMS->send();
                }
            }
        }

        return true;
    }

    public function convertAddressSMSTextMessage($addressObject, $unconvertedSmsMessage)
    {

        $firstName        = $addressObject->getField("firstname")->getValue();
        $lastName         = $addressObject->getField("lastname")->getValue();
        $customerStreet   = $addressObject->getField("address1")->getValue();
        $customerStreet   .= ' ' . $addressObject->getField("address2")->getValue();
        $customerCity     = ' ' . $addressObject->getField("city")->getValue();
        $customerPostcode = $addressObject->getField("postcode")->getValue();

        $convertedMessage = str_replace("{{IMIE}}", $firstName, $unconvertedSmsMessage);
        $convertedMessage = str_replace("{{NAZWISKO}}", $lastName, $convertedMessage);
        $convertedMessage = str_replace("{{ULICA}}", $customerStreet, $convertedMessage);
        $convertedMessage = str_replace("{{KOD}}", $customerPostcode, $convertedMessage);
        $convertedMessage = str_replace("{{MIEJSCOWOSC}}", $customerCity, $convertedMessage);
        $convertedMessage = str_replace("{{NR_ZAMOWIENIA}}", '', $convertedMessage);
        $convertedMessage = str_replace("{{WARTOSC_ZAMOWIENIA}}", '', $convertedMessage);

        return $this->sanitize($convertedMessage);
    }


    public function getAddressChangeEventData()
    {

        $sql = new DbQuery();
        $sql->select('*');
        $sql->from('playsms_events');
        $sql->where('associated_hook="userAddressChanged"');
        $getAddressChangedEvents = Db::getInstance()->executeS($sql);

        return $getAddressChangedEvents;
    }

    public function convertSMSTextMessage($order, $unconvertedSmsMessage)
    {

        $customer          = new Customer($order->id_customer);
        $firstName         = $customer->firstname;
        $lastName          = $customer->lastname;
        $orderTotal        = $order->total_paid;
        $orderTotalRounded = number_format((float)$orderTotal, 2, ',', ' ');
        $order_reference   = $order->reference;
        $addressObject     = new Address($order->id_address_delivery);
        $customerStreet    = $addressObject->address1;
        $customerPostcode  = $addressObject->postcode;
        $customerCity      = $addressObject->city;

        $convertedMessage = str_replace("{{IMIE}}", $firstName, $unconvertedSmsMessage);
        $convertedMessage = str_replace("{{NAZWISKO}}", $lastName, $convertedMessage);
        $convertedMessage = str_replace("{{ULICA}}", $customerStreet, $convertedMessage);
        $convertedMessage = str_replace("{{KOD}}", $customerPostcode, $convertedMessage);
        $convertedMessage = str_replace("{{MIEJSCOWOSC}}", $customerCity, $convertedMessage);
        $convertedMessage = str_replace("{{NR_ZAMOWIENIA}}", $order_reference, $convertedMessage);
        $convertedMessage = str_replace("{{WARTOSC_ZAMOWIENIA}}", $orderTotalRounded, $convertedMessage);

        return $this->sanitize($convertedMessage);
    }

    public function getNewOrderStatus()
    {

        $sql = new DbQuery();
        $sql->select('*');
        $sql->from('playsms_events');
        $sql->where('associated_hook="newOrder"');
        $getNewOrderEvents = Db::getInstance()->executeS($sql);

        return $getNewOrderEvents;
    }

    public function getCorrespondingToCurrentOrderStatusEvent($orderStatusID)
    {

        $sql = new DbQuery();
        $sql->select('*');
        $sql->from('playsms_events');
        $sql->where('associated_hook=' . $orderStatusID);
        $correspondingToCurrentOrderStatusEvents = Db::getInstance()->executeS($sql);

        return $correspondingToCurrentOrderStatusEvents;
    }

    public function getAvailableOrderStates()
    {

        $id_lang = $this->context->language->id;
        $sql     = new DbQuery();
        $sql->select('os.id_order_state, osl.name');
        $sql->from('order_state', 'os');
        $sql->leftJoin(
            'order_state_lang',
            'osl',
            'os.id_order_state = osl.id_order_state AND osl.id_lang =' . (int)$id_lang
        );
        $getAvailableOrderStates = Db::getInstance()->executeS($sql);

        if (empty($getAvailableOrderStates)) {
            return $this->logError($this->l('Nie udało się pobrać dostępnych statusów.'));
        } else {
            $orderStatuses = array();

            foreach ($getAvailableOrderStates as $available_order_status_id) {
                $orderStatuses[$available_order_status_id['id_order_state']] = $available_order_status_id['name'];
            }
        }

        return $orderStatuses;
    }

    private function sanitize($data)
    {

        $data = trim($data);
        $data = Tools::stripslashes($data);
        $data = htmlspecialchars($data);

        return $data;
    }

    private function validateUUIDv4($value)
    {

        if (is_string($value)) {
            $UUIDv4 = '/^[0-9A-F]{8}-[0-9A-F]{4}-4[0-9A-F]{3}-[89AB][0-9A-F]{3}-[0-9A-F]{12}$/i';
            if (preg_match($UUIDv4, $value)) {
                $response = true;
            } else {
                $response = false;
            }
        } else {
            $response = false;
        }

        return $response;
    }
}
